#!/usr/bin/env python3
import unittest
import base64
import subprocess
import tempfile
import sys
import os

class TestCli(unittest.TestCase):
    def test_cli_list(self):
        subprocess.run([sys.executable, '-m', 'pypcode', '-l'], check=True)

    def test_cli_trans(self):
        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.write(base64.b64decode('McA5xnYRSInBg+EfigwKMAwHSP/A6+vD'))
        tf.close()
        try:
            subprocess.run([sys.executable, '-m', 'pypcode', 'x86:LE:64:default', tf.name], check=True)
        finally:
            os.unlink(tf.name)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import os.path
import platform
import struct
import subprocess
import platform
from distutils.command.build_ext import build_ext

ROOT_DIR = os.path.abspath(os.path.dirname(__file__))
LIB_SRC_DIR = os.path.join(ROOT_DIR, 'pypcode', 'native')
SLEIGH_BUILD_DIR = os.path.join(LIB_SRC_DIR, 'build')

class FfiPreBuildExtension(build_ext):
    def pre_run(self, ext, ffi):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError('Please install CMake to build')

        cmake_config_args = [
            '-DCMAKE_VERBOSE_MAKEFILE:BOOL=ON',
            '-DCMAKE_WINDOWS_EXPORT_ALL_SYMBOLS=TRUE',
            ]
        cmake_build_args = []
        if platform.system() == 'Windows':
            is_64b = (struct.calcsize("P")*8 == 64)
            cmake_config_args += ['-A', 'x64' if is_64b else 'Win32']
            cmake_build_args += ['--config', 'Release']

        # Build sleigh and csleigh library
        subprocess.check_call(['cmake', '-S', '.', '-B', 'build'] + cmake_config_args, cwd=LIB_SRC_DIR)
        subprocess.check_call(['cmake', '--build', 'build', '--parallel', '--verbose'] + cmake_build_args, cwd=LIB_SRC_DIR)

        # Build sla files
        bin_ext = {
          'Windows' : '.exe',
          'Linux'   : '',
          'Darwin'  : ''
        }[platform.system()]
        sleigh_bin = os.path.join(SLEIGH_BUILD_DIR, 'bin', 'sleigh' + bin_ext)
        specfiles_dir = os.path.join(self.build_lib, 'pypcode', 'processors')
        subprocess.check_call([sleigh_bin, '-a', specfiles_dir])

        ffi.cdef(open(os.path.join(SLEIGH_BUILD_DIR, "csleigh.i")).read())

def ffibuilder():
    from cffi import FFI
    ffi = FFI()
    LIBS = {
        'Windows': [],
        'Darwin': ['c++'],
        'Linux': ['stdc++']
    }[platform.system()]
    ffi.set_source("pypcode._csleigh",
        """
        #include "build/csleigh.i"
        """,
        libraries=['csleigh'] + LIBS,
        include_dirs=[LIB_SRC_DIR],
        library_dirs=[os.path.join(SLEIGH_BUILD_DIR, 'lib')])
    return ffi

if __name__ == "__main__":
    ffibuilder().compile(verbose=True)



#!/usr/bin/env python3
import os
from setuptools import setup, find_packages
import sys

sys.path.append(os.getcwd())
import build_cffi

def add_pkg_data_dirs(pkg, dirs):
	pkg_data = []
	for d in dirs:
		for root, _, files in os.walk(os.path.join(pkg, d)):
			r = os.path.relpath(root, pkg)
			pkg_data.extend([os.path.join(r, f) for f in files])
	return pkg_data

with open('README.md') as f:
	long_description = f.read()

setup(name='pypcode',
	version='1.0.4',
	description='Python bindings to Ghidra\'s SLEIGH library',
	long_description=long_description,
	long_description_content_type='text/markdown',
	author='Matt Borgerson',
	author_email='contact@mborgerson.com',
	url='https://github.com/angr/pypcode',
	packages=['pypcode'],
	package_data={'pypcode': add_pkg_data_dirs('pypcode', ['processors', 'docs'])},
	setup_requires=['cffi'],
	install_requires=['cffi'],
	cffi_modules=['build_cffi.py:ffibuilder'],
	cmdclass={'build_ext': build_cffi.FfiPreBuildExtension},
	python_requires='>=3.6'
	)
